<?php

namespace Drupal\protected_download\Controller;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\image\Entity\ImageStyle;
use Drupal\protected_download\PolicyRepositoryInterface;
use Drupal\protected_download\SecurityKeyInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Defines a controller to server protected downloads.
 */
class DownloadController extends ControllerBase
{

  /**
   * The download security key service.
   *
   * @var \Drupal\protected_download\SecurityKeyInterface
   */
  protected $securityKey;

  /**
   * The protected download policy repository.
   *
   * @var \Drupal\protected_download\PolicyRepositoryInterface
   */
  protected $policyRepository;

  /**
   * The request time.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The http kernel
   *
   * @var \Symfony\Component\HttpKernel\HttpKernelInterface
   */
  protected $httpKernel;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('protected_download.security_key'),
      $container->get('protected_download.policy_repository'),
      $container->get('datetime.time'),
      $container->get('http_kernel')
    );
  }

  /**
   * Constructs a new protected download controller
   */
  public function __construct(
    SecurityKeyInterface $security_key,
    PolicyRepositoryInterface $policy_repository,
    TimeInterface $time,
    HttpKernelInterface $http_kernel
  ) {
    $this->securityKey = $security_key;
    $this->policyRepository = $policy_repository;
    $this->time = $time;
    $this->httpKernel = $http_kernel;
  }

  /**
   * Verifies an incoming request and delivers a HMAC protected file.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   * @param string $scheme
   *   The URI scheme.
   * @param string $expire
   *   A hex-encoded UNIX timestamp.
   * @param string $hmac
   *   The message authentication code.
   */
  public function deliver(
    Request $request,
    string $scheme,
    string $expire,
    string $hmac
  ) {
    /**
     * Resulting response.
     *
     * @var \Symfony\Component\HttpFoundation\Response
     */
    $response = null;

    // Construct the internal URI and attempt to deliver the file.
    $target = $request->query->get('file');
    $uri = $scheme . '://' . $target;
    if ($this->securityKey->verify($uri, $expire, $hmac)) {

      $stat = @stat($uri);
      if (is_array($stat)) {
        $response = new BinaryFileResponse($uri, 200);

        // Use the same ETag format as Apache.
        $filemtime = $stat['mtime'];
        $filesize = $stat['size'];
        $etag = '"' . dechex($stat['ino']) . '-' . dechex($filesize) . '-' . dechex($filemtime) . '"';
        $response->setEtag($etag);

        // Adapt max-age to remaining time according to the expiry timestamp
        $response->setMaxAge(hexdec($expire) - $this->time->getRequestTime());

        // Set response to public/private depending on settings for scheme.
        if ($this->policyRepository->getPolicy($scheme)->isPrivate()) {
          $response->setPrivate();
        }

        // Enforce http cache validation.
        // FIXME: Check whether this is really necessary.
        if ($response->isNotModified($request)) {
          $response->setNotModified();
        }
      }
      else {
        // If no file exists, attempt to forward to image style download
        // controller.
        [$prefix, $styleId, $scheme, $rest] = explode("/", $target, 4);
        if ($prefix === 'styles') {
          $styleEntity = ImageStyle::load($styleId);
          if ($styleEntity) {
            $query = ['file' => $rest];

            if (!\Drupal::config('image.settings')->get('suppress_itok_output')) {
              $itok = $styleEntity->getPathToken($scheme . "://" . $rest);
              $query[IMAGE_DERIVATIVE_TOKEN] = $itok;
            }

            $response = $this->forward(
              $request,
              'Drupal\image\Controller\ImageStyleDownloadController::deliver',
              [
                'scheme' => $scheme,
                'image_style' => $styleEntity,
              ],
              $query,
            );
          }
        }
      }
    }

    if (isset($response)) {
      return $response;
    } else {
      throw new NotFoundHttpException();
    }
  }

  /**
   * Forwards the request to another controller.
   *
   * Adapted from Symfony\Bundle\FrameworkBundle\Controller\AbstractController::forward()
   */
  protected function forward(
    Request $request,
    string $controller,
    array $path = [],
    array $query = []
  ): Response {
    $path['_controller'] = $controller;
    $subRequest = $request->duplicate($query, null, $path);

    return $this->httpKernel->handle($subRequest, HttpKernelInterface::SUB_REQUEST);
  }

}
