<?php

namespace Drupal\protected_download;

use Drupal\Component\Datetime\Time;
use Drupal\Component\Utility\Crypt;
use Drupal\Core\Site\Settings;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;

/**
 * Provides methods to generate and check the HMAC for protected downloads
 */
class SecurityKey implements SecurityKeyInterface
{

  /**
   * HMAC key.
   *
   * @var string
   */
  protected $hmacKey;

  /**
   * The request time.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The filesystem service.
   *
   * @var \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface
   */
  protected $streamWrapperManager;

  /**
   * Constructs a download security key service.
   *
   * @param \Drupal\Component\Datetime\TimeInterface
   *   The request time service.
   * @param \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface $stream_wrapper_manager
   *   The stream wrapper manager service.
   */
  function __construct(
    Time $time,
    StreamWrapperManagerInterface $stream_wrapper_manager,
    $hmac_key = NULL
  ) {
    $this->hmacKey = $hmac_key ?? Settings::get('protected_download_private_key', \Drupal::service('private_key')->get() . Settings::getHashSalt());
    $this->time = $time;
    $this->streamWrapperManager = $stream_wrapper_manager;
  }

  /**
   * {@inheritdoc}
   */
  function verify(string $uri, string $expire, string $hmac) {
    $result = FALSE;

    if (strlen($uri) && strlen($expire) && strlen($hmac)) {
      $calculated_hmac = Crypt::hmacBase64($uri . $expire, $this->hmacKey);
      $scheme = $this->streamWrapperManager::getScheme($uri);
      $result = hash_equals($calculated_hmac, $hmac) &&
      $this->time->getRequestTime() <= hexdec($expire) &&
        $this->streamWrapperManager->isValidScheme($scheme);
    }

    return $result;
  }

  /**
   * {@inheritdoc}
   */
  function generate(string $uri, string $expire) {
    return Crypt::hmacBase64($uri . $expire, $this->hmacKey);
  }
}
