<?php

namespace Drupal\Tests\protected_download\Functional;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Url;
use Drupal\file\Entity\File;
use Drupal\protected_download\ExactTtlPolicy;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\TestFileCreationTrait;

/**
 * Tests token integration.
 *
 * @group protected_download
 */
class TokenTest extends BrowserTestBase {

  use TestFileCreationTrait {
    getTestFiles as drupalGetTestFiles;
  }

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['file', 'protected_download'];

  /**
   * {@inheritdoc}
   */
  protected $profile = 'testing';

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * The URI for a generated private file.
   *
   * @var string
   */
  protected $privateFileUri;

  /**
   * A file in the private directory.
   *
   * @var \Drupal\file\FileInterface
   */
  protected $privateFile;

  /**
   * {@inheritdoc}
   */
  public function setUp(): void{
    parent::setUp(array('protected_download'));

    /** @var \Drupal\Core\File\FileSystemInterface */
    $fileSystem = \Drupal::service('file_system');

    /** @var \Drupal\file\FileRepositoryInterface */
    $fileRepository = \Drupal::service('file.repository');

    // Prepare a private file.
    $file = File::create([
      'uri' => $this->drupalGetTestFiles('text')[0]->uri,
    ]);
    $file->save();

    $privateFileDir = 'private://' . $this->randomMachineName(8) . '/';
    $fileSystem->prepareDirectory($privateFileDir, FileSystemInterface::CREATE_DIRECTORY);
    $this->privateFileUri = $privateFileDir . $file->getFilename();
    $this->privateFile = $fileRepository->move($file, $this->privateFileUri);
  }

  /**
   * Tests token integration.
   */
  public function testTokenIntegration() {
    $tokenService = \Drupal::token();
    $now = \Drupal::time()->getRequestTime();

    /** @var \Drupal\Core\Datetime\DateFormatterInterface $date_formatter */
    $dateFormatter = \Drupal::service('date.formatter');

    $text = 'Your file is accessible until [file:protected-download-expire] using the following link [file:protected-download-url]';
    $pattern = '#^Your file is accessible until (.*) using the following link (.*)$#';
    $expectedDate = $now + ExactTtlPolicy::DEFAULT_EXACT_TTL;
    $expectedBase = Url::fromUri('base:/protected-download/private')->setAbsolute()->toString();

    $result = $tokenService->replace($text, ['file' => $this->privateFile]);
    $status = preg_match($pattern, $result, $matches);
    $this->assertEquals(1, $status);
    $this->assertSame($dateFormatter->format($expectedDate, 'medium'), $matches[1]);
    $this->assertSame(0, strpos($matches[2], $expectedBase));

    $text = 'Your file is accessible until [file:protected-download-expire:long] using the following link [file:protected-download-url]';
    $result = $tokenService->replace($text, array('file' => $this->privateFile));
    $status = preg_match($pattern, $result, $matches);
    $this->assertEquals(1, $status);
    $this->assertSame($dateFormatter->format($expectedDate, 'long'), $matches[1]);
    $this->assertSame(0, strpos($matches[2], $expectedBase));
  }

}
